using Unitful
using DataFrames
using Plots
# using Gadfly
using UnitfulRecipes
using Roots
using CSV
using MonteCarloMeasurements


unsafe_comparisons(true)

# Tank https://www.amazon.com/Empire-Paintball-BASICS-Pressure-Compressed/dp/B07B6M48SR/
V = (85 ± 5)u"inch^3"
P0 = (4200.0 ± 300)u"psi"
Wtank = (2.3 ± 0.2)u"lb"
Pmax = (100 ± 5)u"psi"

Wsolenoid = 1.5u"kg"

# Params
d_nozzle = ((1 // 16) ± 0.001)u"inch"
a_nozzle = (pi / 4) * d_nozzle^2
Poutmax = (800 ± 50)u"psi"

# Universal Stuff
P_amb = (1 ± 0.2)u"atm"
γ = 1.4 ± 0.05
R = 287.05u"J/(kg * K)"
T = (300 ± 20)u"K"

# Setting up while loop
t = 0.0u"s"
P = P0 |> u"Pa"
M = V * (P / (R * T)) |> u"kg"
# df = DataFrame(Thrust = (0 ± 0)u"N", Pressure = P, Time = t, Mass = M)
df = DataFrame()
ts = 10u"ms"
while M > 0.1u"kg"
    # while t < 30u"s"
    # Calculate what is leaving tank
    P = minimum([P, Pmax])
    ve = sqrt((2 * γ / (γ - 1)) * R * T * (1 - P_amb / P)^((γ - 1) / γ)) |> u"m/s"
    ρ = P / (R * T) |> u"kg/m^3"
    ṁ = ρ * a_nozzle * ve |> u"kg/s"

    Thrust = ṁ * ve + a_nozzle * (P - P_amb) |> u"N"

    # Calculate what is still in the tank
    M = M - ṁ * ts |> u"kg"
    P = (M * R * T) / V |> u"Pa"
    # df_step = DataFrame(Thrust = Thrust, Pressure = P, Time = t, Mass = M)

    if df == DataFrame()
        df = DataFrame(Thrust = Thrust, Pressure = P, Time = t, Mass = M)
    else
        append!(df, DataFrame(Thrust = Thrust, Pressure = P, Time = t, Mass = M))
    end
    t = t + ts
end
impulse = sum(df.Thrust) * ts |> u"N*s"
println("---------------------------\n\n\n\n")
println("Total Impulse: ", impulse)
println("Burn Time: ", t)
plot(impulse)
println("Mass Total: ", Wtank + Wsolenoid + maximum(df.Mass))

# plot(sum(df.Thrust) * ts |> u"N*s")
# print(describe(df))
# df.Time = round(u"s", df.Time)
# plot(df.Time, df.Thrust, title = "Thrust Over Time")
# CSV.write("thrustdata.csv", df .|> ustrip)
# plot(df.Time .|> u"s" .|> ustrip, df.Thrust .|> u"N" .|> ustrip)> ustrip, df.Thrust .|> u"N" .|> ustrip)